from datetime import datetime

from django.conf import settings
from django.core.management.base import BaseCommand

from prometheus_client import (
    CollectorRegistry,
    Counter,
    Gauge,
    push_to_gateway,
)
from prometheus_client.exposition import (
    default_handler,
    basic_auth_handler,
)


class PushgatewayCommand(BaseCommand):

    def __init__(self):
        assert self.job_name
        self.gateway_url = getattr(
            settings,
            'PUSHGATEWAY_URL',
            'http://localhost:9091',
        )
        self.gateway_user = getattr(settings, 'PUSHGATEWAY_USER', None)
        self.gateway_password = getattr(settings, 'PUSHGATEWAY_PASSWORD', None)
        if self.gateway_user and self.gateway_password:
            self.handler = self.auth_handler
        else:
            self.handler = default_handler
        self.registry = CollectorRegistry()
        self.duration = Gauge(
            'job_last_duration_seconds',
            'Last execution duration of a batch job',
            registry=self.registry,
        )
        self.start_datetime = datetime.now()
        super(PushgatewayCommand, self).__init__()

    def auth_handler(self, url, method, timeout, headers, data):
        return basic_auth_handler(
            url=url,
            method=method,
            timeout=timeout,
            headers=headers,
            data=data,
            username=self.gateway_user,
            password=self.gateway_password,
        )

    def gauge(self, name, description, labels=[]):
        return Gauge(name, description, labels, registry=self.registry)

    def counter(self, name, description, labels=[]):
        return Counter(name, description, labels, registry=self.registry)

    def push_metrics(self):
        self.duration.set(
            (datetime.now() - self.start_datetime).total_seconds()
        )
        push_to_gateway(
            self.gateway_url,
            job=self.job_name,
            registry=self.registry,
            handler=self.handler,
        )



import django
import psycopg2.extensions

from django_prometheus.db.common import DatabaseWrapperMixin, \
    ExportingCursorWrapper

if django.VERSION >= (1, 9):
    from django.db.backends.postgresql import base
else:
    from django.db.backends.postgresql_psycopg2 import base


class DatabaseFeatures(base.DatabaseFeatures):
    """Our database has the exact same features as the base one."""
    pass


class DatabaseWrapper(DatabaseWrapperMixin, base.DatabaseWrapper):
    def get_connection_params(self):
        conn_params = super(DatabaseWrapper, self).get_connection_params()
        conn_params['cursor_factory'] = ExportingCursorWrapper(
            psycopg2.extensions.cursor,
            self.alias,
            self.vendor,
        )
        return conn_params

    def create_cursor(self, name=None):
        # cursor_factory is a kwarg to connect() so restore create_cursor()'s
        # default behavior
        if django.VERSION >= (1, 11, 0):
            return base.DatabaseWrapper.create_cursor(self, name=name)
        else:
            return base.DatabaseWrapper.create_cursor(self)



from __future__ import absolute_import
from django_memcached_consul import memcached
from django_prometheus.cache.metrics import (
    django_cache_get_total, django_cache_hits_total, django_cache_misses_total)


class MemcachedCache(memcached.MemcachedCache):
    """Inherit django_memcached_consul to add metrics about hit/miss ratio"""

    def get(self, key, default=None, version=None):
        django_cache_get_total.labels(backend='django_memcached_consul').inc()
        cached = super(MemcachedCache, self).get(
            key, default=None, version=version)
        if cached is not None:
            django_cache_hits_total.labels(
                backend='django_memcached_consul').inc()
        else:
            django_cache_misses_total.labels(
                backend='django_memcached_consul').inc()
        return cached or default



from prometheus_client import Counter

django_cache_get_total = Counter(
    'django_cache_get_total', 'Total get requests on cache', ['backend']
)
django_cache_hits_total = Counter(
    'django_cache_get_hits_total', 'Total hits on cache', ['backend']
)
django_cache_misses_total = Counter(
    'django_cache_get_misses_total', 'Total misses on cache', ['backend']
)
django_cache_get_fail_total = Counter(
    'django_cache_get_fail_total',
    'Total get request failures by cache', ['backend']
)



from django_prometheus.db.common import (
    DatabaseWrapperMixin, ExportingCursorWrapper)
from django.db.backends.mysql import base


class DatabaseFeatures(base.DatabaseFeatures):
    """Our database has the exact same features as the base one."""
    pass


class DatabaseWrapper(DatabaseWrapperMixin, base.DatabaseWrapper):
    CURSOR_CLASS = base.CursorWrapper

    def create_cursor(self, name=None):
        cursor = self.connection.cursor()
        CursorWrapper = ExportingCursorWrapper(
            self.CURSOR_CLASS, self.alias, self.vendor)
        return CursorWrapper(cursor)



import os
from prometheus_client import values


# Override pid function if we have a reusable gunicorn worker ID
if os.environ.get('prometheus_multiproc_dir', None):  # noqa
    try:
        values.ValueClass = values.MultiProcessValue(
            process_identifier=lambda: os.environ.get('APP_WORKER_ID', 1000),
        )
    except TypeError:
        values.ValueClass = values.MultiProcessValue(
            _pidFunc=lambda: os.environ.get('APP_WORKER_ID', 1000),
        )


# Import all files that define metrics. This has the effect that
# `import django_prometheus` will always instanciate all metric
# objects right away.
import django_prometheus.middleware
import django_prometheus.models

# Import pip_prometheus to export the pip metrics automatically.
try:
    import pip_prometheus
except ImportError:
    # If people don't have pip, don't export anything.
    pass

try:  # Load celery exporter if possible
    import django_prometheus.celery
except ImportError:
    pass

default_app_config = 'django_prometheus.apps.DjangoPrometheusConfig'



#!/usr/bin/env python
import socket
import unittest

from mock import patch, call, ANY, MagicMock

from django_prometheus.exports import SetupPrometheusEndpointOnPortRange


class ExportTest(unittest.TestCase):
    @patch('django_prometheus.exports.HTTPServer')
    def testPortRange(self, httpserver_mock):
        httpserver_mock.side_effect = [socket.error, MagicMock()]
        port_range = [8000, 8001]
        SetupPrometheusEndpointOnPortRange(port_range)

        expected_calls = [
            call(('', 8000), ANY),
            call(('', 8001), ANY),
        ]
        self.assertEqual(httpserver_mock.mock_calls, expected_calls)


if __name__ == 'main':
    unittest.main()



from django_prometheus.db.common import DatabaseWrapperMixin
from django.db.backends.sqlite3 import base


class DatabaseFeatures(base.DatabaseFeatures):
    """Our database has the exact same features as the base one."""
    pass


class DatabaseWrapper(DatabaseWrapperMixin, base.DatabaseWrapper):
    CURSOR_CLASS = base.SQLiteCursorWrapper



import django
from django_prometheus import exports

if django.VERSION < (1, 4, 0):
    from django.conf.urls.defaults import url
else:
    from django.conf.urls import url


urlpatterns = [
    url(r'^metrics$', exports.ExportToDjangoView,
        name='prometheus-django-metrics'),
]



from django_redis import cache, exceptions

from django_prometheus.cache.metrics import (
    django_cache_get_fail_total,
    django_cache_get_total,
    django_cache_hits_total,
    django_cache_misses_total,
)


class RedisCache(cache.RedisCache):
    """Inherit redis to add metrics about hit/miss/interruption ratio"""

    @cache.omit_exception
    def get(self, key, default=None, version=None, client=None):
        try:
            django_cache_get_total.labels(backend='redis').inc()
            cached = self.client.get(
                key,
                default=None,
                version=version,
                client=client
            )
        except exceptions.ConnectionInterrupted as e:
            django_cache_get_fail_total.labels(backend='redis').inc()
            if cache.DJANGO_REDIS_IGNORE_EXCEPTIONS or self._ignore_exceptions:
                if cache.DJANGO_REDIS_LOG_IGNORED_EXCEPTIONS:
                    cache.logger.error(str(e))
                return default
            raise
        else:
            if cached is not None:
                django_cache_hits_total.labels(backend='redis').inc()
            else:
                django_cache_misses_total.labels(backend='redis').inc()
            return cached or default



import django
from prometheus_client import Counter, Histogram

from django_prometheus.utils import Time, TimeSince, PowersOf

if django.VERSION >= (1, 10, 0):
    from django.utils.deprecation import MiddlewareMixin
else:
    MiddlewareMixin = object

if django.VERSION < (1, 5, 0):
    from django.core.urlresolvers import resolve


ACCEPTABLE_HTTP_METHODS = (
    'connect',
    'delete',
    'get',
    'head',
    'options',
    'patch',
    'post',
    'put',
    'trace',
)


http_request_duration = Histogram(
    'django_http_request_duration_seconds',
    ('Histogram of requests processing time (including middlewares).'),
    ['handler'],
    buckets=(
        .005, .01, .025, .05, .075, .1, .25, .5, .75, 1.0,
        2.5, 5.0, 7.5, 10.0, 15.0, 30.0, 60.0, 120.0, 300.0
    ),
)
http_request_size = Histogram(
    'django_http_request_size_bytes',
    'Histogram of requests by body size.',
    buckets=PowersOf(2, 30),
)
http_view_duration = Histogram(
    'django_view_duration_seconds',
    'Histogram of view processing duration.',
    ['handler'],
    buckets=(
        .005, .01, .025, .05, .075, .1, .25, .5, .75, 1.0,
        2.5, 5.0, 7.5, 10.0, 15.0, 30.0, 60.0, 120.0, 300.0
    ),
)
http_template_responses = Counter(
    'django_http_template_responses',
    'Count of template responses.',
    ['template_name'],
)
http_responses = Counter(
    'django_http_responses',
    'Count of HTTP responses.',
    ['code', 'handler', 'method'],
)
http_middleware_responses = Counter(
    'django_http_middleware_responses',
    'Count of HTTP responses originating from middlewares and not the view.',
    ['code', 'handler', 'method'],
)
http_response_size = Histogram(
    'django_http_response_size_bytes',
    'Histogram of responses by body size.',
    buckets=PowersOf(2, 30),
)
http_streaming_responses = Counter(
    'django_http_streaming_responses',
    'Count of streaming responses.',
)
exceptions_by_view = Counter(
    'django_exceptions',
    'Count of exceptions.',
    ['handler', 'method'],
)


def request_view_name(request):
    view_name = '<unnamed view>'
    if hasattr(request, 'resolver_match'):
        if request.resolver_match is not None:
            if request.resolver_match.view_name is not None:
                view_name = request.resolver_match.view_name
    elif django.VERSION < (1, 5, 0) and hasattr(request, 'path'):
        try:
            view_name = resolve(request.path).url_name
        except:  # No view match
            pass
    return view_name


def request_method(request):
    method = request.method.lower()
    if method not in ACCEPTABLE_HTTP_METHODS:
        return 'invalid'
    return method


class PrometheusBeforeMiddleware(MiddlewareMixin):

    """Monitoring middleware that should run before other middlewares."""
    def process_request(self, request):
        request.prometheus_before_middleware_event = Time()
        content_length = int(request.META.get('CONTENT_LENGTH') or 0)
        http_request_size.observe(content_length)

    def process_response(self, request, response):
        if hasattr(request, 'prometheus_before_middleware_event'):
            http_request_duration.labels(
                handler=request_view_name(request),
            ).observe(
                TimeSince(request.prometheus_before_middleware_event)
            )
        return response


class PrometheusAfterMiddleware(MiddlewareMixin):

    """Monitoring middleware that should run after other middlewares."""

    def process_request(self, request):
        request.prometheus_after_middleware_event = Time()

    def process_template_response(self, request, response):
        if hasattr(response, 'template_name'):
            http_template_responses.labels(
                template_name=str(response.template_name),
            ).inc()
        return response

    def process_response(self, request, response):
        code = str(response.status_code)
        handler = request_view_name(request)
        method = request_method(request)
        http_responses.labels(
            code=code,
            handler=handler,
            method=method,
        ).inc()
        if hasattr(response, 'streaming') and response.streaming:
            http_streaming_responses.inc()
        if hasattr(response, 'content'):
            http_response_size.observe(len(response.content))

        if hasattr(request, 'prometheus_after_middleware_event'):
            http_view_duration.labels(
                handler=handler,
            ).observe(TimeSince(request.prometheus_after_middleware_event))
        else:
            # In this case it means a middleware responded before the view
            http_middleware_responses.labels(
                code=code,
                handler=handler,
                method=method,
            ).inc()
        return response

    def process_exception(self, request, exception):
        handler = request_view_name(request)
        method = request_method(request)
        exceptions_by_view.labels(handler=handler, method=method).inc()
        return None



from prometheus_client import Counter


connections_total = Counter(
    'django_db_new_connections_total',
    'Counter of created connections by database and by vendor.',
    ['alias', 'vendor'])

connection_errors_total = Counter(
    'django_db_new_connection_errors_total',
    'Counter of connection failures by database and by vendor.',
    ['alias', 'vendor'])

execute_total = Counter(
    'django_db_execute_total',
    ('Counter of executed statements by database and by vendor, including'
     ' bulk executions.'),
    ['alias', 'vendor'])


execute_many_total = Counter(
    'django_db_execute_many_total',
    ('Counter of executed statements in bulk operations by database and'
     ' by vendor.'),
    ['alias', 'vendor'])


errors_total = Counter(
    'django_db_errors_total',
    ('Counter of execution errors by database, vendor and exception type.'),
    ['alias', 'vendor', 'type'])



from django.core.cache.backends import locmem
from django_prometheus.cache.metrics import (
    django_cache_get_total, django_cache_hits_total, django_cache_misses_total)


class LocMemCache(locmem.LocMemCache):
    """Inherit filebased cache to add metrics about hit/miss ratio"""

    def get(self, key, default=None, version=None):
        django_cache_get_total.labels(backend='locmem').inc()
        cached = super(LocMemCache, self).get(
            key, default=None, version=version)
        if cached is not None:
            django_cache_hits_total.labels(backend='locmem').inc()
        else:
            django_cache_misses_total.labels(backend='locmem').inc()
        return cached or default



from __future__ import absolute_import, unicode_literals
from datetime import datetime
import os

from billiard import current_process
from celery.signals import (
    after_task_publish,
    task_postrun,
    task_prerun,
    task_retry,
    worker_process_init,
    worker_process_shutdown,
    celeryd_init,
)
from prometheus_client import values, multiprocess, Counter, Histogram


WORKER_ID_OFFSET = 1001  # 1000 is Celery main process
task_start_time = None  # Used to measure task execution time

tasks_published = Counter(
    'celery_tasks_published',
    'Count of published tasks',
    ['task'],
)
tasks_retried = Counter(
    'celery_tasks_retried',
    'Count of retried tasks',
    ['task'],
)
tasks_executed = Counter(
    'celery_tasks_executed',
    'Count of executed (finished) tasks',
    ['task', 'state'],
)
task_duration = Histogram(
    'celery_task_duration_seconds',
    'Duration of task execution in seconds',
    ['task', 'state'],
)


@after_task_publish.connect
def on_after_task_publish(sender=None, headers=None, body=None, **kwargs):
    """Dispatched when a task has been sent to the broker.
    Note that this is executed in the process that sent the task.
    """
    try:
        task_name = headers['task']
    except:  # headers['task'] is not always available
        task_name = 'UNKNOWN'
    tasks_published.labels(task=task_name).inc()


@task_prerun.connect
def on_task_prerun(*args, **kwargs):
    """Dispatched before a task is executed."""
    global task_start_time
    task_start_time = datetime.now()


@task_postrun.connect
def on_task_postrun(task, state, **kwargs):
    """Dispatched after a task has been executed."""
    task_state = state.lower()
    task_execution_time = (datetime.now() - task_start_time).total_seconds()
    task_name = task.__module__ + "." + task.__name__

    task_duration.labels(
        task=task_name,
        state=task_state,
    ).observe(task_execution_time)
    tasks_executed.labels(task=task_name, state=task_state).inc()


@task_retry.connect
def on_task_retry(sender=None, **kwargs):
    """Dispatched when a task will be retried."""
    task_name = sender.__module__ + "." + sender.__name__  # sender is task
    tasks_retried.labels(task=task_name).inc()


@worker_process_init.connect
def on_worker_process_init(*args, **kwargs):
    """Make use of stable worker IDs to name the dbfiles."""
    values.ValueClass = values.MultiProcessValue(
        _pidFunc=lambda: current_process().index + WORKER_ID_OFFSET,
    )


@worker_process_shutdown.connect
def on_worker_process_shutdown(pid, exitcode, **kwargs):
    """Dispatched in all pool child processes just before they exit.
    We delete the dbfile at that time.
    """
    multiprocess.mark_process_dead(current_process().index + WORKER_ID_OFFSET)



import os
from setuptools import setup, find_packages

LONG_DESCRIPTION = """django-exporter

This library contains code to expose some monitoring metrics relevant
to Django internals so they can be monitored by Prometheus.

See https://github.com/prezi/django-exporter for usage instructions.
"""

setup(
    name="django-exporter",
    version="2.2.4",
    author="David Guerrero",
    author_email="david.guerrero@prezi.com",
    description="Export Django metrics for Prometheus.",
    license="Apache",
    keywords="django monitoring prometheus",
    url="http://github.com/prezi/django-exporter",
    packages=find_packages(),
    test_suite="django_prometheus.tests",
    long_description=LONG_DESCRIPTION,
    install_requires=[
        "django-redis>=4.8,<=4.10.0",
        "prometheus_client>=0.5.0,<=0.9.0"
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.9",
        "Framework :: Django",
        "Topic :: System :: Monitoring",
        "License :: OSI Approved :: Apache Software License",
    ],
)



#!/usr/bin/env python
from django_prometheus.utils import PowersOf
import unittest


class DjangoPrometheusTest(unittest.TestCase):
    def testPowersOf(self):
        """Tests utils.PowersOf."""
        self.assertEqual(
            [0, 1, 2, 4, 8],
            PowersOf(2, 4))
        self.assertEqual(
            [0, 3, 9, 27, 81, 243],
            PowersOf(3, 5, lower=1))
        self.assertEqual(
            [1, 2, 4, 8],
            PowersOf(2, 4, include_zero=False))
        self.assertEqual(
            [4, 8, 16, 32, 64, 128],
            PowersOf(2, 6, lower=2, include_zero=False))


if __name__ == 'main':
    unittest.main()



from django.db import connections
from django.db.backends.dummy.base import DatabaseWrapper
from prometheus_client import Gauge

unapplied_migrations = Gauge(
    'django_migrations_unapplied_total',
    'Count of unapplied migrations by database connection',
    ['connection'])

applied_migrations = Gauge(
    'django_migrations_applied_total',
    'Count of applied migrations by database connection',
    ['connection'])


def ExportMigrationsForDatabase(alias, executor):
    plan = executor.migration_plan(executor.loader.graph.leaf_nodes())
    unapplied_migrations.labels(alias).set(len(plan))
    applied_migrations.labels(alias).set(len(
        executor.loader.applied_migrations))


def ExportMigrations():
    """Exports counts of unapplied migrations.

    This is meant to be called during app startup, ideally by
    django_prometheus.apps.AppConfig.
    """

    # Import MigrationExecutor lazily. MigrationExecutor checks at
    # import time that the apps are ready, and they are not when
    # django_prometheus is imported. ExportMigrations() should be
    # called in AppConfig.ready(), which signals that all apps are
    # ready.
    from django.db.migrations.executor import MigrationExecutor

    if 'default' in connections and (
            type(connections['default']) == DatabaseWrapper):
        # This is the case where DATABASES = {} in the configuration,
        # i.e. the user is not using any databases. Django "helpfully"
        # adds a dummy database and then throws when you try to
        # actually use it. So we don't do anything, because trying to
        # export stats would crash the app on startup.
        return
    for alias in connections.databases:
        executor = MigrationExecutor(connections[alias])
        ExportMigrationsForDatabase(alias, executor)
